/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.manager.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import io.swagger.annotations.*;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;

import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.manager.dto.SaveUserInfoByProjectDTO;
import org.springblade.manager.dto.WbsTreeContractDTO;
import org.springblade.manager.dto.ProjectInfoDTO;
import org.springblade.manager.entity.ContractInfo;
import org.springblade.manager.entity.WbsTreePrivate;
import org.springblade.manager.service.IContractInfoService;
import org.springblade.manager.service.IWbsTreePrivateService;
import org.springblade.manager.service.IWbsTreeService;
import org.springblade.manager.vo.*;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import org.springblade.manager.entity.ProjectInfo;
import org.springblade.manager.wrapper.ProjectInfoWrapper;
import org.springblade.manager.service.IProjectInfoService;
import org.springblade.core.boot.ctrl.BladeController;

import java.util.List;

/**
 * 客户级项目表 控制器
 *
 * @author liuyc
 * @since 2022-04-13
 */
@RestController
@AllArgsConstructor
@RequestMapping("/projectInfo")
@Api(value = "项目管理", tags = "项目管理接口")
public class ProjectInfoController extends BladeController {

    private final IProjectInfoService projectInfoService;
    private final IWbsTreeService wbsTreeService;
    private final IWbsTreePrivateService wbsTreePrivateService;
    private final IContractInfoService iContractInfoService;

    /**
     * 详情
     */
    @GetMapping("/detail")
    @ApiOperationSupport(order = 1)
    @ApiOperation(value = "详情", notes = "传入projectInfoId")
    @ApiImplicitParam(value = "id", name = "id", required = true)
    public R<ProjectInfoVO> detail(ProjectInfo projectInfo) {
        ProjectInfo detail = projectInfoService.getOne(projectInfo.getId());
        return R.data(ProjectInfoWrapper.build().entityVO(detail));
    }


    /**
     * 详情
     */
    @GetMapping("/detail2")
    @ApiOperationSupport(order = 1)
    @ApiOperation(value = "详情2", notes = "传入projectInfoId、ReferenceWbsTemplateId、ReferenceWbsTemplateType")
    @ApiImplicitParams({
            @ApiImplicitParam(value = "id", name = "id", required = true),
            @ApiImplicitParam(value = "ReferenceWbsTemplateId", name = "ReferenceWbsTemplateId", required = true),
            @ApiImplicitParam(value = "ReferenceWbsTemplateType", name = "ReferenceWbsTemplateType", required = true)
    })
    public R<ProjectInfoVO2> detail2(ProjectInfo projectInfo) {
        ProjectInfoVO2 detail = projectInfoService.getOneByCondition(projectInfo.getId(),
                projectInfo.getReferenceWbsTemplateId(),
                projectInfo.getReferenceWbsTemplateType());
        return R.data(detail);
    }


    /**
     * 分页 客户级项目表
     */
    @GetMapping("/list")
    @ApiOperationSupport(order = 2)
    @ApiOperation(value = "分页", notes = "传入projectInfo")
    public R<IPage<ProjectInfoVO>> list(ProjectInfo projectInfo, Query query) {
        IPage<ProjectInfo> pages = projectInfoService.page(Condition.getPage(query), Condition.getQueryWrapper(projectInfo));

        return R.data(ProjectInfoWrapper.build().pageVO(pages));
    }


    /**
     * 自定义分页 客户级项目表
     */
    @GetMapping("/page")
    @ApiOperationSupport(order = 3)
    @ApiOperation(value = "自定义分页", notes = "传入projectInfo")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "current", value = "当前页", required = true),
            @ApiImplicitParam(name = "size", value = "每页的数量", required = true)
    })
    public R<IPage<ProjectInfoVO>> page(ProjectInfoVO projectInfo, Query query) {
        IPage<ProjectInfoVO> pages = projectInfoService.selectProjectInfoPage(Condition.getPage(query), projectInfo);
        return R.data(pages);
    }


    /**
     * 新增 客户级项目表
     */
    @PostMapping("/save")
    @ApiOperationSupport(order = 4)
    @ApiOperation(value = "新增", notes = "传入projectInfo")
    public R save(@Valid @RequestBody ProjectInfoDTO projectInfo) {
        return R.status(projectInfoService.save(projectInfo));
    }

    /**
     * 修改 客户级项目表
     */
    @PostMapping("/update")
    @ApiOperationSupport(order = 5)
    @ApiOperation(value = "修改", notes = "传入projectInfo")
    public R update(@Valid @RequestBody ProjectInfo projectInfo) {
        return R.status(projectInfoService.updateById(projectInfo));
    }

    /**
     * 新增或修改 客户级项目表
     */
    @PostMapping("/submit")
    @ApiOperationSupport(order = 6)
    @ApiOperation(value = "新增或修改", notes = "传入projectInfo")
    @ApiImplicitParam(value = "id", name = "id", required = true)
    public R submit(@Valid @RequestBody ProjectInfo projectInfo) {
        boolean b = projectInfoService.saveOrUpdate(projectInfo);
        if (b) {
            return R.data(projectInfo, "操作成功");
        }
        return R.fail("操作失败");
    }


    /**
     * 删除 客户级项目表
     */
    @PostMapping("/remove")
    @ApiOperationSupport(order = 7)
    @ApiOperation(value = "逻辑删除", notes = "传入ids")
    public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
        List<ContractInfo> infos = iContractInfoService.findContractInProject(ids);
        if (infos.size() > 0) {
            return R.fail("当前项目有合同段,删除失败");
        }
        List<WbsTreePrivate> wbsTreePrivates = wbsTreePrivateService.findWbsTreePrivate(ids);
        if (wbsTreePrivates.size() > 0) {
            return R.fail("当前项目存在WBS私有树,删除失败");
        }
        return R.status(projectInfoService.deleteLogic(Func.toLongList(ids)));
    }

    /**
     * 查询业务条数
     *
     * @return
     */
    @GetMapping("/listContractInfoCount")
    @ApiOperationSupport(order = 8)
    @ApiOperation(value = "统计合同数", notes = "null")
    public List<ContractlnfoCountVO> selectContractInfoCount() {
        return projectInfoService.selectContractInfoCount();
    }

    /**
     * 项目私有库分配wbs树下拉框列表type=1质检 =2实验
     */
    @GetMapping("/findWbsTreeList")
    @ApiOperationSupport(order = 9)
    @ApiOperation(value = "项目合同段私有库分配wbs树下拉框列表", notes = "null")
    public R<WbsTreeAllListVO> findWbsTreeList(Integer type) {
        WbsTreeAllListVO list = wbsTreeService.findWbsTreeList(type);
        if (list != null) {
            return R.data(list);
        }
        return R.fail("未查询到信息");
    }

    /**
     * 项目、合同段私有库wbs节点Tree加载
     * （项目私有wbs设置界面，左侧加载可用,
     * 项目分配wbs初始化时,右侧展示当前项目id的私有节点,左侧则展示的是分配wbs树保存后引用的哪个项目id下的私有树
     * 合同段分配wbs时进入默认加载左侧节点树可用）
     */
    @GetMapping("/tree")
    @ApiOperationSupport(order = 10)
    @ApiOperation(value = "查询项目私有wbs节点树形结构（多个接口复用）", notes = "wbsId、项目id")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "projectId", value = "项目id", required = true),
            @ApiImplicitParam(name = "wbsId", value = "wbsId", required = true)
    })
    public R<List<WbsTreePrivateVO>> tree(String wbsId, String projectId) {
        List<WbsTreePrivateVO> tree = wbsTreePrivateService.tree(wbsId, projectId);
        if (tree != null && tree.size() > 0) {
            return R.data(tree);
        }
        return R.fail(200, "未查询到信息");
    }


    /**
     * 保存或修改项目合同分配wbs树
     */
    @PostMapping("/submitWbsTreeInProject")
    @ApiOperationSupport(order = 11)
    @ApiOperation(value = "保存或修改项目合同分配wbs树", notes = "传入节点ids,wbsId(如果引用是其他私有库,那就是该库的项目id),项目id,合同段id")
    public R submitWbsTreeInProject(@RequestBody WbsTreeContractDTO pawDTO) {
        String s = wbsTreeService.submitWbsTreeInProject(pawDTO);
        switch (s) {
            case "1":
                return R.success("公有wbs树新元素表同步成功");
            case "2":
                return R.success("关联wbs树成功");
            default:
                return R.success("关联wbs树失败");
        }

    }

    /**
     * 根据用户id查询项目合同岗位列表信息(用于展示在用户编辑、详情中)
     */
    @GetMapping("/findProjectAndContractList")
    @ApiOperationSupport(order = 12)
    @ApiOperation(value = "根据用户id查询项目合同岗位列表信息", notes = "传入用户id")
    public R<List<SaveUserInfoByProjectVO2>> findProjectAndContractList(Long userId) {
        List<SaveUserInfoByProjectVO2> result = wbsTreeService.findProjectAndContractList(userId);
        return R.data(result);
    }
}
